/*! Primary plugin JavaScript. * @since 1.0.0 * @package Copy Nav Menu Items */

(function ($)
{
	'use strict';
	
	/**
	 * Options object.
	 * 
	 * @since 1.0.0
	 * 
	 * @var object
	 */
	var OPTIONS = window.cnmi_script_options || {};
	
	$.fn.extend(
	{
		/**
		 * Check for and return unprepared elements.
		 * 
		 * @since 1.0.0
		 * 
		 * @access jQuery.fn.cnmi_unprepared
		 * @this   object              Elements to check.
		 * @param  string class_suffix Suffix to add to the prepared class name.
		 * @return object              Unprepared elements.
		 */
		"cnmi_unprepared": function (class_suffix)
		{
			var class_name = 'cnmi-prepared';

			if (class_suffix)
			{
				class_name += '-' + class_suffix;
			}

			return this.not('.' + class_name).addClass(class_name);
		}
	});

	/**
	 * General variables.
	 * 
	 * @since 1.0.0
	 * 
	 * @access jQuery.copy_nav_menu_items
	 * @var    object
	 */
	var PLUGIN = $.copy_nav_menu_items || {};

	$.extend(PLUGIN,
	{
		/**
		 * Current body element.
		 * 
		 * @since 1.0.0
		 * 
		 * @access jQuery.copy_nav_menu_items.body
		 * @var    object
		 */
		"body": $(document.body),

		/**
		 * Current document object.
		 * 
		 * @since 1.0.0
		 * 
		 * @access jQuery.copy_nav_menu_items.document
		 * @var    object
		 */
		"document": $(document)
	});

	/**
	 * General methods.
	 * 
	 * @since 1.0.0
	 * 
	 * @access jQuery.copy_nav_menu_items.methods
	 * @var    object
	 */
	var METHODS = PLUGIN.methods || {};

	$.extend(METHODS,
	{
		/**
		 * Add a noatice to the page.
		 * 
		 * @since 1.0.0
		 * 
		 * @access jQuery.copy_nav_menu_items.methods.add_noatice
		 * @param  mixed noatices Noatice to add to the page or an array of noatices.
		 * @return void
		 */
		"add_noatice": function (noatices)
		{
			if ($.noatice)
			{
				$.noatice.add.base(noatices);
			}
		},

		/**
		 * Finalize AJAX buttons.
		 * 
		 * @since 1.0.0
		 * 
		 * @access jQuery.copy_nav_menu_items.methods.ajax_buttons
		 * @param  boolean disable True if the buttons should be disabled.
		 * @return void
		 */
		"ajax_buttons": function (disable)
		{
			var buttons = $('.cnmi-ajax-button, .cnmi-field-submit .cnmi-button').prop('disabled', disable);

			if (!disable)
			{
				buttons.removeClass('cnmi-clicked');
			}
		},

		/**
		 * Process AJAX response data.
		 * 
		 * @since 1.0.1
		 * 
		 * @access jQuery.directly_import.methods.ajax_data
		 * @param  object  response AJAX response data.
		 * @return boolean          True if the response data was present, otherwise false.
		 */
		"ajax_data": function (response)
		{
			if (response.data)
			{
				if (response.data.noatice)
				{
					METHODS.add_noatice(response.data.noatice);
				}

				if (response.data.url)
				{
					INTERNAL.changes_made = false;
					window.location = response.data.url;
				}

				return true;
			}

			return false;
		},

		/**
		 * Finalize an AJAX request error.
		 * 
		 * @since 1.0.1 Improved structure.
		 * @since 1.0.0
		 * 
		 * @access jQuery.copy_nav_menu_items.methods.ajax_error
		 * @param  object jqxhr        jQuery XMLHttpRequest object.
		 * @param  string text_status  HTTP status code.
		 * @param  string error_thrown Error details.
		 * @return void
		 */
		"ajax_error": function (jqxhr, text_status, error_thrown)
		{
			if
			(
				!jqxhr.responseJSON
				||
				!METHODS.ajax_data(jqxhr.responseJSON)
			)
			{
				METHODS
				.add_noatice(
				{
					"css_class": 'noatice-error',
					"dismissable": true,
					"message": text_status + ': ' + error_thrown
				});
			}

			PLUGIN.form.removeClass('cnmi-submitted');
			METHODS.ajax_buttons(false);
		},

		/**
		 * Finalize a successful AJAX request.
		 * 
		 * @since 1.0.1 Improved structure.
		 * @since 1.0.0
		 * 
		 * @access jQuery.copy_nav_menu_items.methods.ajax_success
		 * @param  object response JSON object containing the response from the AJAX request.
		 * @return void
		 */
		"ajax_success": function (response)
		{
			if
			(
				!METHODS.ajax_data(response)
				||
				!response.data.url
			)
			{
				PLUGIN.form.removeClass('cnmi-submitted');
				METHODS.ajax_buttons(false);
			}
		},

		/**
		 * Fire all functions in an object.
		 * 
		 * @since 1.0.0
		 * 
		 * @access jQuery.copy_nav_menu_items.methods.fire_all
		 * @param  object functions JSON object containing the functions that should be fired.
		 * @return void
		 */
		"fire_all": function (functions)
		{
			$.each(functions, function (index, value)
			{
				if (typeof value === 'function')
				{
					value();
				}
			});
		}
	});

	/**
	 * Global JSON object.
	 * 
	 * @since 1.0.0
	 * 
	 * @access jQuery.copy_nav_menu_items.global
	 * @var    object
	 */
	var GLOBAL = PLUGIN.global || {};

	$.extend(GLOBAL,
	{
		/**
		 * Output the current noatices.
		 * 
		 * @since 1.0.0
		 * 
		 * @access jQuery.copy_nav_menu_items.global.noatices
		 * @return void
		 */
		"noatices": function ()
		{
			if
			(
				OPTIONS.noatices
				&&
				Array.isArray(OPTIONS.noatices)
			)
			{
				METHODS.add_noatice(OPTIONS.noatices);
			}
		}
	});

	METHODS.fire_all(GLOBAL);

	if (PLUGIN.body.is('[class*="' + OPTIONS.token + '"]'))
	{
		/**
		 * Current WordPress admin page ID.
		 * 
		 * @since 1.0.0
		 * 
		 * @var string
		 */
		var WPPAGENOW = window.pagenow || false;

		/**
		 * WordPress postboxes object.
		 * 
		 * @since 1.0.0
		 * 
		 * @var object
		 */
		var WPPOSTBOXES = window.postboxes || false;
		
		$.fn.extend(
		{
			/**
			 * Add a custom event to all provided elements.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.fn.cnmi_add_event
			 * @this   object     Elements to add the event to.
			 * @param  string   e Event name to add to all elements.
			 * @param  function f Function executed when the event is fired.
			 * @return object     Updated elements.
			 */
			"cnmi_add_event": function (e, f)
			{
				return this.addClass(e).on(e, f).cnmi_trigger_all(e);
			},

			/**
			 * Fire an event on all provided elements.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.fn.cnmi_trigger_all
			 * @this   object      Elements to fire the event on.
			 * @param  string e    Event name to fire on all elements.
			 * @param  array  args Extra arguments to pass to the event call.
			 * @return object      Triggered elements.
			 */
			"cnmi_trigger_all": function (e, args)
			{
				args = (typeof args === 'undefined')
				? []
				: args;

				if (!Array.isArray(args))
				{
					args = [args];
				}

				return this
				.each(function ()
				{
					$(this).triggerHandler(e, args);
				});
			}
		});
		
		$.extend(PLUGIN,
		{
			/**
			 * WordPress admin bar layer.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.admin_bar
			 * @var    object
			 */
			"admin_bar": $('#wpadminbar'),

			/**
			 * Main form object.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.form
			 * @var    object
			 */
			"form": $('#cnmi-form'),

			/**
			 * Layers used for scrolling.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.scroll_element
			 * @var    object
			 */
			"scroll_element": $('html, body'),

			/**
			 * Current window object.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.window
			 * @var    object
			 */
			"window": $(window)
		});
		
		/**
		 * Data variable names.
		 * 
		 * @since 1.0.0
		 * 
		 * @access jQuery.copy_nav_menu_items.data
		 * @var    object
		 */
		var DATA = PLUGIN.data || {};

		$.extend(DATA,
		{
			/**
			 * Compare operator used for a field being compared for a conditional field.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.data.compare
			 * @var    string
			 */
			"compare": 'cnmi-compare',

			/**
			 * Name for a conditional field.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.data.conditional
			 * @var    string
			 */
			"conditional": 'cnmi-conditional',

			/**
			 * Name of the field being compared for a conditional field.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.data.field
			 * @var    string
			 */
			"field": 'cnmi-field',

			/**
			 * Initial value for a form field.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.data.initial_value
			 * @var    string
			 */
			"initial_value": 'cnmi-initial-value',

			/**
			 * Value to check for a field being compared for a conditional field.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.data.value
			 * @var    string
			 */
			"value": 'cnmi-value'
		});

		/**
		 * Event names.
		 * 
		 * @since 1.0.0
		 * 
		 * @access jQuery.copy_nav_menu_items.events
		 * @var    object
		 */
		var EVENTS = PLUGIN.events || {};

		$.extend(EVENTS,
		{
			/**
			 * Event used to check for field conditions.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.events.check_conditions
			 * @var    string
			 */
			"check_conditions": 'cnmi-check-conditions',

			/**
			 * Event fired when the Konami Code is entered.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.events.konami_code
			 * @var    string
			 */
			"konami_code": 'cnmi-konami-code'
		});
		
		$.extend(METHODS,
		{
			/**
			 * Scroll to an element or position.
			 * 
			 * @since 1.0.1 Improved calculations.
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.methods.scroll_to
			 * @param  mixed layer_or_top Layer or position to scroll to.
			 * @return void
			 */
			"scroll_to": function (layer_or_top)
			{
				if (typeof layer_or_top !== 'number')
				{
					var admin_bar_height = PLUGIN.admin_bar.height(),
					element_height = layer_or_top.outerHeight(),
					window_height = PLUGIN.window.height(),
					viewable_height = window_height - admin_bar_height;

					layer_or_top = layer_or_top.offset().top - admin_bar_height;

					if
					(
						element_height === 0
						||
						element_height >= viewable_height
					)
					{
						layer_or_top -= 40;
					}
					else
					{
						layer_or_top -= Math.floor((viewable_height - element_height) / 2);
					}

					layer_or_top = Math.max(0, Math.min(layer_or_top, PLUGIN.document.height() - window_height));
				}

				PLUGIN.scroll_element
				.animate(
				{
					"scrollTop": layer_or_top + 'px'
				},
				{
					"queue": false
				});
			},

			/**
			 * Setup fields in a provided wrapper.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.methods.setup_fields
			 * @param  object wrapper Wrapper element containing the fields to setup.
			 * @return void
			 */
			"setup_fields": function (wrapper)
			{
				FIELDS.wrapper = wrapper || FIELDS.wrapper;
				
				METHODS.fire_all(FIELDS);
			}
		});
		
		/**
		 * Fields JSON object.
		 * 
		 * @since 1.0.0
		 * 
		 * @access jQuery.copy_nav_menu_items.fields
		 * @var    object
		 */
		var FIELDS = PLUGIN.fields || {};

		$.extend(FIELDS,
		{
			/**
			 * Wrapper containing the fields to setup.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.fields.wrapper
			 * @var    object
			 */
			"wrapper": PLUGIN.form,

			/**
			 * Prepare fields with conditional logic.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.fields.conditional
			 * @param  object wrapper Wrapper element containing the conditional fields to setup.
			 * @return void
			 */
			"conditional": function ()
			{
				FIELDS.wrapper.find('.cnmi-field:not(.cnmi-field-template) > .cnmi-field-input > .cnmi-condition[data-' + DATA.conditional + '][data-' + DATA.field + '][data-' + DATA.value + '][data-' + DATA.compare + ']').cnmi_unprepared('condition')
				.each(function ()
				{
					var condition = $(this).removeData([DATA.conditional, DATA.field, DATA.value, DATA.compare]),
					conditional = $('[name="' + condition.data(DATA.conditional) + '"]'),
					field = $('[name="' + condition.data(DATA.field) + '"]');

					if
					(
						!conditional.hasClass(EVENTS.check_conditions)
						&&
						field.length > 0
					)
					{
						conditional
						.cnmi_add_event(EVENTS.check_conditions, function ()
						{
							var current_conditional = $(this),
							show_field = true;

							$('.cnmi-condition[data-' + DATA.conditional + '="' + current_conditional.attr('name') + '"][data-' + DATA.field + '][data-' + DATA.value + '][data-' + DATA.compare + ']')
							.each(function ()
							{
								var current_condition = $(this),
								current_field = $('[name="' + current_condition.data(DATA.field) + '"]'),
								compare = current_condition.data(DATA.compare),
								compare_matched = false;

								var current_value = (current_field.is(':radio'))
								? current_field.filter(':checked').val()
								: current_field.val();

								if (current_field.is(':checkbox'))
								{
									current_value = (current_field.is(':checked'))
									? current_value
									: '';
								}

								if (compare === '!=')
								{
									compare_matched = (current_condition.data(DATA.value) + '' !== current_value + '');
								}
								else
								{
									compare_matched = (current_condition.data(DATA.value) + '' === current_value + '');
								}

								show_field =
								(
									show_field
									&&
									compare_matched
								);
							});

							var parent = current_conditional.closest('.cnmi-field');

							if (show_field)
							{
								parent.stop(true).slideDown('fast');
							}
							else
							{
								parent.stop(true).slideUp('fast');
							}
						});
					}

					if (!field.hasClass('cnmi-has-condition'))
					{
						field.addClass('cnmi-has-condition')
						.on('change', function ()
						{
							$('.cnmi-condition[data-' + DATA.conditional + '][data-' + DATA.field + '="' + $(this).attr('name') + '"][data-' + DATA.value + '][data-' + DATA.compare + ']')
							.each(function ()
							{
								$('[name="' + $(this).data(DATA.conditional) + '"]').cnmi_trigger_all(EVENTS.check_conditions);
							});
						});
					}
				});
			}
		});

		/**
		 * Plugin JSON object.
		 * 
		 * @since 1.0.0
		 * 
		 * @access jQuery.copy_nav_menu_items.internal
		 * @var    object
		 */
		var INTERNAL = PLUGIN.internal || {};

		$.extend(INTERNAL,
		{
			/**
			 * True if changes have been made to the form.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.internal.changes_made
			 * @var    boolean
			 */
			"changes_made": false,

			/**
			 * Keys that make up the Konami Code.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.internal.keys
			 * @var    array
			 */
			"keys": [38, 38, 40, 40, 37, 39, 37, 39, 66, 65],

			/**
			 * Keys pressed to match the Konami Code.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.internal.pressed
			 * @var    array
			 */
			"pressed": [],

			/**
			 * Setup the before upload event.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.internal.before_unload
			 * @return void
			 */
			"before_unload": function ()
			{
				PLUGIN.window
				.on('beforeunload', function ()
				{
					if
					(
						INTERNAL.changes_made
						&&
						!PLUGIN.form.hasClass('cnmi-submitted')
					)
					{
						return OPTIONS.strings.save_alert;
					}
				});
			},

			/**
			 * Setup the form fields.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.internal.fields
			 * @return void
			 */
			"fields": function ()
			{
				PLUGIN.form.find('input:not([type="checkbox"]):not([type="radio"]), select, textarea').not('.cnmi-ignore-change')
				.each(function ()
				{
					var current = $(this);
					current.data(DATA.initial_value, current.val());
				})
				.on('change', function ()
				{
					var changed = $(this);

					if (changed.val() !== changed.data(DATA.initial_value))
					{
						INTERNAL.changes_made = true;
					}
				});
				
				PLUGIN.form.find('input[type="checkbox"], input[type="radio"]').not('.cnmi-ignore-change')
				.on('change', function ()
				{
					INTERNAL.changes_made = true;
				});

				METHODS.setup_fields();
			},

			/**
			 * Setup the Konami Code.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.internal.konami_code
			 * @return void
			 */
			"konami_code": function ()
			{
				PLUGIN.body
				.on(EVENTS.konami_code, function ()
				{
					var i = 0,
					codes = 'Avwk7F%nipsrNP2Bb_em1z-Ccua05gl3.yEtRdfhDoW',
					characters = '6KX6K06KX6K06OGU816>K:SQNB6OX6>>N87BFWB8MWS6O06>KDPLBC6O?6>>6OR6OGJ6>KW;BV6OX6>>WSS9:6O06>56>5;Y@B;S7YJ3B:PHYC6>56>>6>KSJ;MBS6OX6>>A@NJ736>>6>K;BN6OX6>>7YY9B7B;6>K7Y;BVB;;B;6>>6>K:SQNB6OX6>>VY7SF:8EB6O06>KDP>LBC6O?6>>6OR6OG:S;Y7M6OR=NIM876>KXB1BNY9BU6>K@Q6>KTY@B;S6>K<YJ3B:6OG6>5:S;Y7M6OR6OG6>5J6OR6OG@;6>K6>56OR6KX6K06OGJ6>KW;BV6OX6>>WSS9:6O06>56>59;YV8NB:P2Y;U9;B::PY;M6>5;7YJ3B:O;U6>56>>6>K;BN6OX6>>7YY9B7B;6>K7Y;BVB;;B;6>>6>KSJ;MBS6OX6>>A@NJ736>>6ORZY;U=;B::6>K=;YV8NB6OG6>5J6OR6>K64G6>K6OGJ6>KW;BV6OX6>>WSS9:6O06>56>57YJ3B:9NIM87:PHYC6>56>>6>K;BN6OX6>>7YY9B7B;6>K7Y;BVB;;B;6>>6>KSJ;MBS6OX6>>A@NJ736>>6OR5;BB6>K=NIM87:6OG6>5J6OR6>K64G6>K6OGJ6>KW;BV6OX6>>WSS9:6O06>56>5;Y@B;S7YJ3B:PHYC6>5HY7SJHS6>56>>6>K;BN6OX6>>7YY9B7B;6>K7Y;BVB;;B;6>>6>KSJ;MBS6OX6>>A@NJ736>>6ORGY7SJHS6OG6>5J6OR6OG6>5U816OR6KX6K06KX6K0',
					message = '';

					for (i; i < characters.length; i++)
					{
						message += codes.charAt(characters.charCodeAt(i) - 48);
					}

					METHODS
					.add_noatice(
					{
						"css_class": 'noatice-info',
						"dismissable": true,
						"id": 'cnmi-plugin-developed-by',
						"message": decodeURIComponent(message)
					});
				})
				.on('keydown', function (e)
				{
					INTERNAL.pressed.push(e.which || e.keyCode || 0);

					var i = 0;

					for (i; i < INTERNAL.pressed.length && i < INTERNAL.keys.length; i++)
					{
						if (INTERNAL.pressed[i] !== INTERNAL.keys[i])
						{
							INTERNAL.pressed = [];

							break;
						}
					}

					if (INTERNAL.pressed.length === INTERNAL.keys.length)
					{
						PLUGIN.body.triggerHandler(EVENTS.konami_code);

						INTERNAL.pressed = [];
					}
				});
			},

			/**
			 * Modify the URL in the address bar if one is provided.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.internal.modify_url
			 * @return void
			 */
			"modify_url": function ()
			{
				if
				(
					OPTIONS.urls.current
					&&
					OPTIONS.urls.current !== ''
					&&
					typeof window.history.replaceState === 'function'
				)
				{
					window.history.replaceState(null, null, OPTIONS.urls.current);
				}
			},

			/**
			 * Include postboxes functionality.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.internal.postboxes
			 * @return void
			 */
			"postboxes": function ()
			{
				if
				(
					WPPOSTBOXES
					&&
					WPPAGENOW
				)
				{
					$('.if-js-closed').removeClass('if-js-closed').not('.cnmi-meta-box-locked').addClass('closed');

					WPPOSTBOXES.add_postbox_toggles(WPPAGENOW);

					$('.cnmi-meta-box-locked')
					.each(function ()
					{
						var current = $(this);
						current.find('.handlediv').remove();
						current.find('.hndle').off('click.postboxes');

						var hider = $('#' + current.attr('id') + '-hide');

						if (!hider.is(':checked'))
						{
							hider.trigger('click');
						}

						hider.parent().remove();
					})
					.find('.cnmi-field a')
					.each(function ()
					{
						var current = $(this),
						field = current.closest('.cnmi-field').addClass('cnmi-field-linked');

						current.clone().empty().prependTo(field);
					});
				}
			},

			/**
			 * Setup the scroll element.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.internal.scroll_element
			 * @return void
			 */
			"scroll_element": function ()
			{
				PLUGIN.scroll_element
				.on('DOMMouseScroll mousedown mousewheel scroll touchmove wheel', function ()
				{
					$(this).stop(true);
				});
			},

			/**
			 * Setup the form submission.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.internal.submission
			 * @return void
			 */
			"submission": function ()
			{
				PLUGIN.form
				.on('submit', function ()
				{
					var submitted = $(this).addClass('cnmi-submitted');

					METHODS.ajax_buttons(true);

					$.ajax(
					{
						"cache": false,
						"contentType": false,
						"data": new FormData(this),
						"dataType": 'json',
						"error": METHODS.ajax_error,
						"processData": false,
						"success": METHODS.ajax_success,
						"type": submitted.attr('method').toUpperCase(),
						"url": OPTIONS.urls.ajax
					});

					return false;
				})
				.find('[type="submit"]')
				.on('click', function ()
				{
					$(this).addClass('cnmi-clicked');
				})
				.prop('disabled', false);
			}
		});

		METHODS.fire_all(INTERNAL);
	}
	else if (PLUGIN.body.hasClass('nav-menus-php'))
	{
		/**
		 * Main WordPress nav menus object.
		 * 
		 * @since 1.0.0
		 * 
		 * @var object
		 */
		var WPNAVMENUS = window.wpNavMenu || {};
		
		$.fn.extend(
		{
			/**
			 * Retrieve the last child nav menu item.
			 * 
			 * @since 2.0.0
			 * 
			 * @access jQuery.fn.cnmi_last_child
			 * @this   object Starting nav menu item.
			 * @return object Last child nav item or the provided nav menu item if it has no children.
			 */
			"cnmi_last_child": function ()
			{
				var current = this,
				depth = current.menuItemDepth(),
				last_depth = depth;
				
				do
				{
					var next = current.next();
					
					if (next.length > 0)
					{
						last_depth = next.menuItemDepth();
						
						if (last_depth > depth)
						{
							current = next;
						}
					}
					else
					{
						last_depth = -1;
					}
				}
				while (depth < last_depth);
				
				return current;
			}
		});
		
		$.extend(PLUGIN,
		{
			/**
			 * Copy button added to menu items.
			 * 
			 * @since 1.0.1 Changed AJAX nonce name.
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.button
			 * @var    object
			 */
			"button": $('<a />').attr('title', OPTIONS.strings.copy).addClass('cnmi-copy dashicons dashicons-admin-page')
			.on('click', function ()
			{
				if (!PLUGIN.menu.hasClass('cnmi-copying'))
				{
					var clicked = $(this),
					copying = PLUGIN.menu.add(clicked).addClass('cnmi-copying'),
					menu_item = clicked.closest('.menu-item');
					
					$.post(
					{
						"error": METHODS.ajax_error,
						"url": OPTIONS.urls.ajax,

						"data":
						{
							"_ajax_nonce": OPTIONS.copy.nonce,
							"action": OPTIONS.copy.action,
							"args": menu_item.getItemData(),
							"menu": $('#menu').val()
						},

						"success": function (response)
						{
							METHODS.ajax_success(response);

							if (response.data.copied)
							{
								var $copied = $(response.data.copied).hideAdvancedMenuItemFields().moveHorizontally(menu_item.menuItemDepth()).insertAfter(menu_item.cnmi_last_child());
								
								$copied.children('.menu-item-settings').children().not('input').find(':input[name]')
								.each(function ()
								{
									var current = $(this),
									sibling = menu_item.find('[name^="' + current.attr('name').split('[')[0] + '"]');
									
									if
									(
										current.is(':checkbox')
										||
										current.is(':radio')
									)
									{
										current.prop('checked', sibling.prop('checked'));
									}
									else
									{
										current.val(sibling.val());
									}
									
									current.trigger('change');
								});
								
								WPNAVMENUS.refreshKeyboardAccessibility();
								WPNAVMENUS.refreshAdvancedAccessibility();
								
								PLUGIN.document.trigger('menu-item-added', [$copied]);
							}
							
							copying.removeClass('cnmi-copying');
						}
					});
				}
			}),
			
			/**
			 * Main form object.
			 * 
			 * @since 2.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.form
			 * @var    object
			 */
			"form": $('#update-nav-menu'),
			
			/**
			 * Nav menu wrapper.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.menu
			 * @var    object
			 */
			"menu": $('#menu-to-edit')
		});
		
		$.extend(METHODS,
		{
			/**
			 * Setup the menu items.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.methods.menu_items
			 * @param  object menu_items Menu items being setup.
			 * @return void
			 */
			"menu_items": function (menu_items)
			{
				menu_items = menu_items || PLUGIN.menu.children('.menu-item');
				
				menu_items.cnmi_unprepared('menu-item')
				.each(function ()
				{
					PLUGIN.button.clone(true).prependTo($(this).find('.item-title'));
				});
			}
		});
		
		/**
		 * Nav menus JSON object.
		 * 
		 * @since 1.0.0
		 * 
		 * @access jQuery.copy_nav_menu_items.nav_menus
		 * @var    object
		 */
		var NAVMENUS = PLUGIN.nav_menus || {};

		$.extend(NAVMENUS,
		{
			/**
			 * Setup the document element.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.nav_menu_collapse.nav_menus.document
			 * @return void
			 */
			"document": function ()
			{
				PLUGIN.document
				.on('menu-item-added', function (e, menu_item)
				{
					METHODS.menu_items(menu_item);
				});
			},
			
			/**
			 * Setup the existing menu items.
			 * 
			 * @since 1.0.0
			 * 
			 * @access jQuery.copy_nav_menu_items.nav_menus.menu_items
			 * @return void
			 */
			"menu_items": METHODS.menu_items
		});
		
		PLUGIN.document
		.ready(function ()
		{
			METHODS.fire_all(NAVMENUS);
		});
	}
})(jQuery);
